///////////////////////////////////////////////////////////////////////////////
//
//  OTTO.H
//
//  This module contains class declarations and defines.
//
//  Author :    Les Thaler
//  Copyright (c) 1996 Irving De La Cruz and Les Thaler
//  

#ifndef _HELLO_MAPI_H_
#define _HELLO_MAPI_H_
#include "resource.h"

#define APPNAME                 "OTTO"
#define MAX_SUBJ                255
#define SZ_REPLY_SUBJ           "Highbrow University"
#define SZ_ERROR_REPLY          "Sorry. Only admissions applications can be sent to this mailbox<EOM>"
#define SZ_BODY                 "Thank you for considering Highbrow, home of the Fighting Neandrethals.Your application has been received and is being processed."

#define REQUIRED_FOLDER_MASK    (FOLDER_IPM_OUTBOX_VALID      |\
                                 FOLDER_IPM_SENTMAIL_VALID    |\
                                 FOLDER_IPM_WASTEBASKET_VALID |\
                                 FOLDER_IPM_SUBTREE_VALID     |\
                                 FOLDER_IPM_INBOX_VALID)

#define PUB_FLAGS                (STORE_MODIFY_OK | STORE_CREATE_OK)

#define LOGON_FLAGS                      (MAPI_ALLOW_OTHERS | MAPI_LOGON_UI)
#define PR_SENT_REPRESENTING_RECIP_TYPE  PR_NULL // saves spot in sender proplist for recip type
#define SZOOF_MSGCLASS                   "IPM.NOTE.RULES.OTTOREPLY"
#define WM_INIT                          (WM_USER + 100)


enum _RECIPS_   {NAME,EID,ADDR,RECIP,EMAIL,SKEY,NUM_RECIP_PROPS};
enum _COLLEGES_ {ADM,ARTS,SCI,BUS,NUM_COLLEGES};
enum _FOLDERS_  {INBOX,OUTBOX,WASTE,SENT,IPM_ROOT,PUB_ADM,NUM_DEF_FOLDERS = PUB_ADM,PUB_ARTS,PUB_SCI,PUB_BUS,NUM_FOLDERS};

#define NUM_PUB_FOLDERS (NUM_FOLDERS - PUB_ADM)

extern char g_szAppName[];
extern char g_szTitle[];
extern LPSTR gszFolderNames[NUM_PUB_FOLDERS];

typedef class CSession * PCSession;
typedef class CAdviseSink * PCAdviseSink;

class CAdviseSink : public IMAPIAdviseSink
{
    public:
        inline CAdviseSink(PCSession pCSess)
        {
            m_cRef = 0;
            m_pCSess = pCSess;
        };

        inline ~CAdviseSink(){};

        MAPI_IUNKNOWN_METHODS(IMPL);
        MAPI_IMAPIADVISESINK_METHODS(IMPL);
    private:
        ULONG       m_cRef;
        PCSession   m_pCSess;
};

class CSession
{
    friend class CAdviseSink;
    public:        
        CSession(LPMAPISESSION pSess, HINSTANCE hInst)
        {
            m_pSess    = pSess;    
            m_hInst    = hInst;

            for (int i = 0; i < NUM_FOLDERS; i++)
                m_pFolders[i] = NULL;

            m_pMDB         = NULL;
            m_pPubMDB      = NULL;
            m_psbFolderEID = NULL;
            m_ulAdviseCnx  = 0;
        };

        ~CSession()
        {
            if (m_ulAdviseCnx)
                m_pMDB -> Unadvise(m_ulAdviseCnx);


            for (int i = 0; i < NUM_FOLDERS; i++)
                if (m_pFolders[i])
                    m_pFolders[i] -> Release();


            if (m_pMDB)
                m_pMDB -> Release();

            if (m_pPubMDB)
                m_pPubMDB -> Release();
                            
            if (m_pSess)
                m_pSess -> Release();

            DeleteCriticalSection(&m_cs);
            MAPIFreeBuffer((LPVOID) m_psbFolderEID);
            MAPIUninitialize();
        };

        HINSTANCE           m_hInst;
        LPSBinary           m_psbFolderEID;
        LPMAPIFOLDER        m_pFolders[NUM_FOLDERS];
        LPMAPISESSION       m_pSess;
        CRITICAL_SECTION    m_cs;

        STDMETHODIMP    Init();
        STDMETHODIMP    OpenDefStore(HWND hWnd);
        STDMETHODIMP    OpenIPMSubtree(HWND hWnd);
        STDMETHODIMP    CreateSubfolders(LPMAPIFOLDER pPubIPM);
        STDMETHODIMP    SetNotifications(LPMAPIADVISESINK pAdviseSink);
        STDMETHODIMP    ValidatePublicFolders();
        void inline     ReleasePubStore()
        {
            if (m_pPubMDB && (m_pMDB != m_pPubMDB))
                m_pPubMDB -> Release();

            m_pPubMDB = NULL;
        }

    private:
        TCHAR               m_szTitle[MAX_PATH];
        LPMDB               m_pMDB,
                            m_pPubMDB;
        ULONG               m_ulAdviseCnx;

        void            RouteToFolder(LPENTRYLIST psba, int iCollege);
        STDMETHODIMP    SendReply(LPMESSAGE pInMsg,int * pCollege);
        STDMETHODIMP    CreateReplyNote(LPMESSAGE   pInMsg,
                                        int       * pCollege,
                                        LPMESSAGE * ppReplyMsg);

};


extern "C"
{
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK About(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK ReplyNote(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
}

BOOL WINAPI      InitApplication(HINSTANCE);
BOOL WINAPI      InitInstance(HINSTANCE hInstance,int nCmdShow);

void WINAPI      InitDialog(HWND hDlg,LPARAM lParam);//,PContext & pContext);
STDMETHODIMP     CopySBinary(LPSBinary psbDest,
                             const LPSBinary psbSrc,
                             LPVOID pParent);
STDMETHODIMP     InitSession(HWND hWnd,
                             HINSTANCE hInst,
                             PCSession * ppCSess);

BOOL             operator == (const SBinary & lhs, const SBinary & rhs);

// Displays a message box and posts a quit message on a MAPI 
// warning or error. Note that use of this macro is intended
// for the duration of a single (Windows) message. If code follows this
// that depends on the result of an earlier call in the same message,
// continued execution may not be possible.

#define OnError(hWnd, hRes)                                         \
{                                                                   \
    HRESULT hr = (hRes);                                            \
    if (S_OK != hr)                                                 \
    {                                                               \
        char szMsg[80];                                             \
        wsprintf(szMsg,"Encountered MAPI error 0x%08X.",hr);        \
        if (IDOK == MessageBox((hWnd),                              \
                                szMsg,                              \
                                "Hit Cancel to Quit, OK to Debug",  \
                                MB_OKCANCEL))                       \
            DebugBreak();                                           \
        else                                                        \
            PostMessage((hWnd),WM_COMMAND,ID_ERROR_EXIT,0);         \
    }                                                               \
} 

BOOL WINAPI BreakToDebug(HRESULT hr, LPMAPIERROR pErr,HWND hWnd);

#define ShowLastError(hRes, hWnd, _pIF_)    \
{                                           \
    HRESULT     hr   = (hRes);              \
    LPMAPIERROR pErr = NULL;                \
    if (hr)                                 \
    {                                       \
        (_pIF_) -> GetLastError(hr,0,&pErr);\
        if (BreakToDebug(hr,pErr,hWnd))     \
            DebugBreak();                   \
        MAPIFreeBuffer((LPVOID)pErr);       \
    }                                       \
}
    
#endif
